/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <crown/allocation.h>

#include <pelib/AmplOutput.hpp>
#include <pelib/AmplInput.hpp>
#include <pelib/Vector.hpp>
#include <pelib/Set.hpp>
#include <pelib/Matrix.hpp>
#include <pelib/GraphML.hpp>
#include <pelib/XMLSchedule.hpp>

using namespace std;
using namespace pelib;

int
main(int argc, char **argv)
{
	Algebra mapp;

	ifstream taskgraph(argv[1], std::ios::in);
	ifstream platform(argv[2], std::ios::in);

	Algebra alg_arch = AmplInput(AmplInput::floatHandlers()).parse(platform);
	Platform arch(alg_arch);
	Algebra sched = AmplOutput(AmplOutput::floatHandlers()).parse(cin);
	Taskgraph *tg = GraphML().parse(taskgraph);
	Algebra input = tg->buildAlgebra(arch);
	taskgraph.close();
	platform.close();

	input = input.merge(arch.buildAlgebra());
	input = input.merge(sched);

	cout << setprecision(6)
	<< setiosflags(ios::fixed)
	<< setiosflags(ios::showpoint);

	Schedule schedule("Generated from Algebra", Schedule::addStartTime(input, *tg, arch));
	XMLSchedule().dump(cout, schedule, *tg, arch);

	delete tg;
	return EXIT_SUCCESS;
}

