/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <crown/allocation.h>

#include <pelib/AmplOutput.hpp>
#include <pelib/AmplInput.hpp>
#include <pelib/Vector.hpp>
#include <pelib/Matrix.hpp>
#include <pelib/GraphML.hpp>

using namespace std;
using namespace pelib;

int
main(int argc, char **argv)
{
	Algebra mapp;

	ifstream taskgraph(argv[1], std::ios::in);
	ifstream platform(argv[2], std::ios::in);
	ifstream parameters(argv[3], std::ios::in);

	Algebra alg_arch = AmplInput(AmplInput::floatHandlers()).parse(platform);
	Platform *arch = new Platform(alg_arch);
	Algebra param = AmplInput(AmplInput::floatHandlers()).parse(parameters);
	Taskgraph *tg = GraphML().parse(taskgraph);
	Algebra input = tg->buildAlgebra(*arch);
	taskgraph.close();
	platform.close();
	parameters.close();

	input = input.merge(arch->buildAlgebra());
	input = input.merge(param);

	delete arch;
	delete tg;

	cout << setprecision(6)
	<< setiosflags(ios::fixed)
	<< setiosflags(ios::showpoint);

	mapp = allocation_fastest(input);

	// Garbage to make output to look like Ampl/Gurobi output
	cout << "Presolve eliminates 0 constraints and 0 variables." << endl;
	cout << "Substitution eliminates 0 variables." << endl;
	cout << "Adjusted problem:" << endl;
	cout << "0 variables:" << endl;
	cout << "	0 binary variables" << endl;
	cout << "	0 linear variable" << endl;
	cout << "0 constraints, all linear; 0 nonzeros" << endl;
	cout << "0 linear objective; 0 nonzero." << endl;
	cout << endl;
	cout << "Gurobi 5.1.0: timelim 300" << endl;
	cout << "Gurobi 5.1.0: heuristic solution; nonoptimal 0" << endl;
	cout << "29 simplex iterations" << endl;
	cout << "plus 1 simplex iteration for intbasis" << endl;

	AmplOutput(AmplOutput::floatHandlers()).dump(cerr, Scalar<float>("complexity", allocation_fastest_complexity(mapp)));

	AmplOutput(AmplOutput::floatHandlers()).dump(cout, mapp);

	return EXIT_SUCCESS;
}

