/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <crown/mapping.h>

#include <pelib/AmplOutput.hpp>
#include <pelib/AmplInput.hpp>
#include <pelib/Vector.hpp>
#include <pelib/Matrix.hpp>

using namespace std;
using namespace pelib;

int
main(int argc, char **argv)
{
	ifstream taskgraph(argv[1], std::ios::in);

	Algebra input = AmplInput(AmplInput::floatHandlers()).parse(taskgraph);
	taskgraph.close();

	cout << setprecision(6)
	<< setiosflags(ios::fixed)
	<< setiosflags(ios::showpoint);

	input = mapping_ltlg(input);

	// Garbage to make output to look like Ampl/Gurobi output
	cout << "Presolve eliminates 0 constraints and 0 variables." << endl;
	cout << "Substitution eliminates 0 variables." << endl;
	cout << "Adjusted problem:" << endl;
	cout << "0 variables:" << endl;
	cout << "	0 binary variables" << endl;
	cout << "	0 linear variable" << endl;
	cout << "0 constraints, all linear; 0 nonzeros" << endl;
	cout << "0 linear objective; 0 nonzero." << endl;
	cout << endl;
	cout << "LTLG heuristic: heuristic solution; nonoptimal 0" << endl;
	cout << "29 simplex iterations" << endl;
	cout << "plus 1 simplex iteration for intbasis" << endl;
		
	float quality = mapping_quality(input);
	Scalar<float> q("quality", quality);
	input.insert(&q);

	AmplOutput(AmplOutput::floatHandlers()).dump(cout, input);
	AmplOutput(AmplOutput::floatHandlers()).dump(cerr, Scalar<float>("complexity", mapping_ltlg_complexity(input)));

	return EXIT_SUCCESS;
}

